@@ -88,6 +88,9 @@ typedef struct
   /* Distance between mouse pointer and the nearby border. Negative when
      scrolling updards.*/
   gint distance;
+
+  GtkTreeModelFilterVisibleFunc custom_filter;
+  gpointer custom_filter_data;
 } EmpathyIndividualViewPriv;
 
 typedef struct
@@ -414,6 +417,7 @@ individual_view_persona_drag_received (GtkWidget *self,
   FolksPersona *persona = NULL;
   const gchar *persona_uid;
   GList *individuals, *l;
+  GeeIterator *iter = NULL;
   gboolean retval = FALSE;
 
   persona_uid = (const gchar *) gtk_selection_data_get_data (selection);
@@ -425,23 +429,28 @@ individual_view_persona_drag_received (GtkWidget *self,
 
   for (l = individuals; l != NULL; l = l->next)
     {
-      GList *personas, *p;
+      GeeSet *personas;
 
       personas = folks_individual_get_personas (FOLKS_INDIVIDUAL (l->data));
-
-      for (p = personas; p != NULL; p = p->next)
+      iter = gee_iterable_iterator (GEE_ITERABLE (personas));
+      while (gee_iterator_next (iter))
         {
-          if (!tp_strdiff (folks_persona_get_uid (FOLKS_PERSONA (p->data)),
-              persona_uid))
+          FolksPersona *persona_cur = gee_iterator_get (iter);
+
+          if (!tp_strdiff (folks_persona_get_uid (persona), persona_uid))
             {
-              persona = g_object_ref (p->data);
+              /* takes ownership of the ref */
+              persona = persona_cur;
               individual = g_object_ref (l->data);
               goto got_persona;
             }
+          g_clear_object (&persona_cur);
         }
+      g_clear_object (&iter);
     }
 
 got_persona:
+  g_clear_object (&iter);
   g_list_free (individuals);
 
   if (persona == NULL || individual == NULL)
@@ -1406,21 +1415,15 @@ individual_view_search_key_navigation_cb (GtkWidget *search,
   GdkEvent *event,
   EmpathyIndividualView *view)
 {
-  GdkEventKey *eventkey = ((GdkEventKey *) event);
+  GdkEvent *new_event;
   gboolean ret = FALSE;
 
-  if (eventkey->keyval == GDK_KEY_Up || eventkey->keyval == GDK_KEY_Down
-      || eventkey->keyval == GDK_KEY_F2)
-    {
-      GdkEvent *new_event;
+  new_event = gdk_event_copy (event);
+  gtk_widget_grab_focus (GTK_WIDGET (view));
+  ret = gtk_widget_event (GTK_WIDGET (view), new_event);
+  gtk_widget_grab_focus (search);
 
-      new_event = gdk_event_copy (event);
-      gtk_widget_grab_focus (GTK_WIDGET (view));
-      ret = gtk_widget_event (GTK_WIDGET (view), new_event);
-      gtk_widget_grab_focus (search);
-
-      gdk_event_free (new_event);
-    }
+  gdk_event_free (new_event);
 
   return ret;
 }
@@ -1689,14 +1692,21 @@ static gboolean
 individual_view_is_visible_individual (EmpathyIndividualView *self,
     FolksIndividual *individual,
     gboolean is_online,
-    gboolean is_searching)
+    gboolean is_searching,
+    const gchar *group,
+    gboolean is_fake_group,
+    guint event_count)
 {
   EmpathyIndividualViewPriv *priv = GET_PRIV (self);
   EmpathyLiveSearch *live = EMPATHY_LIVE_SEARCH (priv->search_widget);
-  const gchar *str;
-  GList *personas, *l;
+  GeeSet *personas;
+  GeeIterator *iter;
   gboolean is_favorite, contains_interesting_persona = FALSE;
 
+  /* Always display individuals having pending events */
+  if (event_count > 0)
+    return TRUE;
+
   /* We're only giving the visibility wrt filtering here, not things like
    * presence. */
   if (priv->show_untrusted == FALSE &&
@@ -1707,56 +1717,59 @@ individual_view_is_visible_individual (EmpathyIndividualView *self,
 
   /* Hide all individuals which consist entirely of uninteresting personas */
   personas = folks_individual_get_personas (individual);
-  for (l = personas; l; l = l->next)
+  iter = gee_iterable_iterator (GEE_ITERABLE (personas));
+  while (!contains_interesting_persona && gee_iterator_next (iter))
     {
-      if (empathy_folks_persona_is_interesting (FOLKS_PERSONA (l->data)))
-        {
-          contains_interesting_persona = TRUE;
-          break;
-        }
+      FolksPersona *persona = gee_iterator_get (iter);
+
+      if (empathy_folks_persona_is_interesting (persona))
+        contains_interesting_persona = TRUE;
+
+      g_clear_object (&persona);
     }
+  g_clear_object (&iter);
 
   if (contains_interesting_persona == FALSE)
     return FALSE;
 
   is_favorite = folks_favourite_details_get_is_favourite (
       FOLKS_FAVOURITE_DETAILS (individual));
-  if (is_searching == FALSE)
-    return (priv->show_offline || is_online || is_favorite);
-
-  /* check alias name */
-  str = folks_alias_details_get_alias (FOLKS_ALIAS_DETAILS (individual));
-
-  if (empathy_live_search_match (live, str))
-    return TRUE;
-
-  /* check contact id, remove the @server.com part */
-  for (l = personas; l; l = l->next)
-    {
-      const gchar *p;
-      gchar *dup_str = NULL;
-      gboolean visible;
-
-      if (!empathy_folks_persona_is_interesting (FOLKS_PERSONA (l->data)))
-        continue;
-
-      str = folks_persona_get_display_id (l->data);
-      p = strstr (str, "@");
-      if (p != NULL)
-        str = dup_str = g_strndup (str, p - str);
-
-      visible = empathy_live_search_match (live, str);
-      g_free (dup_str);
-      if (visible)
+  if (is_searching == FALSE) {
+    if (is_favorite && is_fake_group &&
+        !tp_strdiff (group, EMPATHY_INDIVIDUAL_STORE_FAVORITE))
+        /* Always display favorite contacts in the favorite group */
         return TRUE;
-    }
 
-  /* FIXME: Add more rules here, we could check phone numbers in
-   * contact's vCard for example. */
+    return (priv->show_offline || is_online);
+  }
 
-  return FALSE;
+  return empathy_individual_match_string (individual,
+      empathy_live_search_get_text (live),
+      empathy_live_search_get_words (live));
 }
 
+static gchar *
+get_group (GtkTreeModel *model,
+    GtkTreeIter *iter,
+    gboolean *is_fake)
+{
+  GtkTreeIter parent_iter;
+  gchar *name = NULL;
+
+  *is_fake = FALSE;
+
+  if (!gtk_tree_model_iter_parent (model, &parent_iter, iter))
+    return NULL;
+
+  gtk_tree_model_get (model, &parent_iter,
+      EMPATHY_INDIVIDUAL_STORE_COL_NAME, &name,
+      EMPATHY_INDIVIDUAL_STORE_COL_IS_FAKE_GROUP, is_fake,
+      -1);
+
+  return name;
+}
+
+
 static gboolean
 individual_view_filter_visible_func (GtkTreeModel *model,
     GtkTreeIter *iter,
@@ -1769,6 +1782,10 @@ individual_view_filter_visible_func (GtkTreeModel *model,
   GtkTreeIter child_iter;
   gboolean visible, is_online;
   gboolean is_searching = TRUE;
+  guint event_count;
+
+  if (priv->custom_filter != NULL)
+    return priv->custom_filter (model, iter, priv->custom_filter_data);
 
   if (priv->search_widget == NULL ||
       !gtk_widget_get_visible (priv->search_widget))
@@ -1779,14 +1796,21 @@ individual_view_filter_visible_func (GtkTreeModel *model,
       EMPATHY_INDIVIDUAL_STORE_COL_IS_SEPARATOR, &is_separator,
       EMPATHY_INDIVIDUAL_STORE_COL_IS_ONLINE, &is_online,
       EMPATHY_INDIVIDUAL_STORE_COL_INDIVIDUAL, &individual,
+      EMPATHY_INDIVIDUAL_STORE_COL_EVENT_COUNT, &event_count,
       -1);
 
   if (individual != NULL)
     {
+      gchar *group;
+      gboolean is_fake_group;
+
+      group = get_group (model, iter, &is_fake_group);
+
       visible = individual_view_is_visible_individual (self, individual,
-          is_online, is_searching);
+          is_online, is_searching, group, is_fake_group, event_count);
 
       g_object_unref (individual);
+      g_free (group);
 
       /* FIXME: Work around bgo#626552/bgo#621076 */
       if (visible == TRUE)
@@ -1809,17 +1833,25 @@ individual_view_filter_visible_func (GtkTreeModel *model,
   for (valid = gtk_tree_model_iter_children (model, &child_iter, iter);
        valid; valid = gtk_tree_model_iter_next (model, &child_iter))
     {
+      gchar *group;
+      gboolean is_fake_group;
+
       gtk_tree_model_get (model, &child_iter,
         EMPATHY_INDIVIDUAL_STORE_COL_INDIVIDUAL, &individual,
         EMPATHY_INDIVIDUAL_STORE_COL_IS_ONLINE, &is_online,
+        EMPATHY_INDIVIDUAL_STORE_COL_EVENT_COUNT, &event_count,
         -1);
 
       if (individual == NULL)
         continue;
 
+      group = get_group (model, &child_iter, &is_fake_group);
+
       visible = individual_view_is_visible_individual (self, individual,
-          is_online, is_searching);
+          is_online, is_searching, group, is_fake_group, event_count);
+
       g_object_unref (individual);
+      g_free (group);
 
       /* show group if it has at least one visible contact in it */
       if (visible == TRUE)
@@ -2424,7 +2456,8 @@ got_avatar (GObject *source_object,
   EmpathyIndividualManager *manager;
   gchar *text;
   GtkWindow *parent;
-  GList *l, *personas;
+  GeeSet *personas;
+  GeeIterator *iter;
   guint persona_count = 0;
   gboolean can_block;
   GError *error = NULL;
@@ -2443,19 +2476,21 @@ got_avatar (GObject *source_object,
    * so we still display the remove dialog. */
 
   personas = folks_individual_get_personas (individual);
+  iter = gee_iterable_iterator (GEE_ITERABLE (personas));
 
   /* If we have more than one TpfPersona, display a different message
    * ensuring the user knows that *all* of the meta-contacts' personas will
    * be removed. */
-  for (l = personas; l != NULL; l = l->next)
+  while (persona_count < 2 && gee_iterator_next (iter))
     {
-      if (!empathy_folks_persona_is_interesting (FOLKS_PERSONA (l->data)))
-        continue;
+      FolksPersona *persona = gee_iterator_get (iter);
 
-      persona_count++;
-      if (persona_count >= 2)
-        break;
+      if (empathy_folks_persona_is_interesting (persona))
+        persona_count++;
+
+      g_clear_object (&persona);
     }
+  g_clear_object (&iter);
 
   if (persona_count < 2)
     {
@@ -2547,10 +2582,15 @@ empathy_individual_view_get_individual_menu (EmpathyIndividualView *view)
   GtkWidget *item;
   GtkWidget *image;
   gboolean can_remove = FALSE;
-  GList *l;
+  GeeSet *personas;
+  GeeIterator *iter;
 
   g_return_val_if_fail (EMPATHY_IS_INDIVIDUAL_VIEW (view), NULL);
 
+  if (priv->individual_features == EMPATHY_INDIVIDUAL_FEATURE_NONE)
+    /* No need to create a context menu */
+    return NULL;
+
   individual = empathy_individual_view_dup_selected (view);
   if (individual == NULL)
     return NULL;
@@ -2559,19 +2599,21 @@ empathy_individual_view_get_individual_menu (EmpathyIndividualView *view)
    * remove. This will act as a best-effort option. If any Personas cannot be
    * removed from the server, then this option will just be inactive upon
    * subsequent menu openings */
-  for (l = folks_individual_get_personas (individual); l != NULL; l = l->next)
+  personas = folks_individual_get_personas (individual);
+  iter = gee_iterable_iterator (GEE_ITERABLE (personas));
+  while (!can_remove && gee_iterator_next (iter))
     {
-      FolksPersona *persona = FOLKS_PERSONA (l->data);
+      FolksPersona *persona = gee_iterator_get (iter);
       FolksPersonaStore *store = folks_persona_get_store (persona);
       FolksMaybeBool maybe_can_remove =
           folks_persona_store_get_can_remove_personas (store);
 
       if (maybe_can_remove == FOLKS_MAYBE_BOOL_TRUE)
-        {
-          can_remove = TRUE;
-          break;
-        }
+        can_remove = TRUE;
+
+      g_clear_object (&persona);
     }
+  g_clear_object (&iter);
 
   menu = empathy_individual_menu_new (individual, priv->individual_features);
 
@@ -2800,3 +2842,22 @@ empathy_individual_view_start_search (EmpathyIndividualView *self)
   else
     gtk_widget_show (GTK_WIDGET (priv->search_widget));
 }
+
+void
+empathy_individual_view_set_custom_filter (EmpathyIndividualView *self,
+    GtkTreeModelFilterVisibleFunc filter,
+    gpointer data)
+{
+  EmpathyIndividualViewPriv *priv = GET_PRIV (self);
+
+  priv->custom_filter = filter;
+  priv->custom_filter_data = data;
+}
+
+void
+empathy_individual_view_refilter (EmpathyIndividualView *self)
+{
+  EmpathyIndividualViewPriv *priv = GET_PRIV (self);
+
+  gtk_tree_model_filter_refilter (priv->filter);
+}
