@@ -142,7 +142,7 @@ persona_active_new (EmpathyPersonaStore *self,
   ShowActiveData *data;
 
   DEBUG ("Contact:'%s' now active, and %s be removed",
-      folks_aliasable_get_alias (FOLKS_ALIASABLE (persona)),
+      folks_alias_details_get_alias (FOLKS_ALIAS_DETAILS (persona)),
       remove_ ? "WILL" : "WILL NOT");
 
   data = g_slice_new0 (ShowActiveData);
@@ -186,12 +186,9 @@ persona_set_active (EmpathyPersonaStore *self,
     gboolean active,
     gboolean set_changed)
 {
-  EmpathyPersonaStorePriv *priv;
   GtkTreePath *path;
   GtkTreeIter iter;
 
-  priv = GET_PRIV (self);
-
   path = find_persona (self, persona);
   if (path == NULL)
     return;
@@ -213,7 +210,7 @@ static gboolean
 persona_active_cb (ShowActiveData *data)
 {
   const gchar *alias =
-      folks_aliasable_get_alias (FOLKS_ALIASABLE (data->persona));
+      folks_alias_details_get_alias (FOLKS_ALIAS_DETAILS (data->persona));
 
   if (data->remove)
     {
@@ -235,7 +232,7 @@ persona_updated_cb (FolksPersona *persona,
     EmpathyPersonaStore *self)
 {
   DEBUG ("Contact:'%s' updated, checking roster is in sync...",
-      folks_aliasable_get_alias (FOLKS_ALIASABLE (persona)));
+      folks_alias_details_get_alias (FOLKS_ALIAS_DETAILS (persona)));
 
   update_persona (self, persona);
 }
@@ -289,7 +286,7 @@ add_persona (EmpathyPersonaStore *self,
 
   priv = GET_PRIV (self);
 
-  alias = folks_aliasable_get_alias (FOLKS_ALIASABLE (persona));
+  alias = folks_alias_details_get_alias (FOLKS_ALIAS_DETAILS (persona));
   if (EMP_STR_EMPTY (alias))
     return;
 
@@ -408,7 +405,7 @@ update_persona (EmpathyPersonaStore *self,
   const gchar *alias;
 
   path = find_persona (self, persona);
-  alias = folks_aliasable_get_alias (FOLKS_ALIASABLE (persona));
+  alias = folks_alias_details_get_alias (FOLKS_ALIAS_DETAILS (persona));
 
   if (path == NULL)
     {
@@ -438,8 +435,8 @@ update_persona (EmpathyPersonaStore *self,
       gtk_tree_path_free (path);
 
       /* Get online state now. */
-      now_online = folks_presence_owner_is_online (
-          FOLKS_PRESENCE_OWNER (persona));
+      now_online = folks_presence_details_is_online (
+          FOLKS_PRESENCE_DETAILS (persona));
 
       /* Get online state before. */
       gtk_tree_model_get (GTK_TREE_MODEL (self), &iter,
@@ -485,11 +482,11 @@ update_persona (EmpathyPersonaStore *self,
           EMPATHY_PERSONA_STORE_COL_DISPLAY_ID,
               folks_persona_get_display_id (persona),
           EMPATHY_PERSONA_STORE_COL_PRESENCE_TYPE,
-              folks_presence_owner_get_presence_type (
-                  FOLKS_PRESENCE_OWNER (persona)),
+              folks_presence_details_get_presence_type (
+                  FOLKS_PRESENCE_DETAILS (persona)),
           EMPATHY_PERSONA_STORE_COL_STATUS,
-              folks_presence_owner_get_presence_message (
-                  FOLKS_PRESENCE_OWNER (persona)),
+              folks_presence_details_get_presence_message (
+                  FOLKS_PRESENCE_DETAILS (persona)),
           EMPATHY_PERSONA_STORE_COL_IS_ONLINE, now_online,
           EMPATHY_PERSONA_STORE_COL_CAN_AUDIO_CALL,
               empathy_contact_get_capabilities (contact) &
@@ -529,11 +526,11 @@ update_persona (EmpathyPersonaStore *self,
 
 static void
 individual_personas_changed_cb (GObject *object,
-    GList *added,
-    GList *removed,
+    GeeSet *added,
+    GeeSet *removed,
     EmpathyPersonaStore *self)
 {
-  GList *l;
+  GeeIterator *iter;
 
   /* One of the personas' row references might hold the last reference to the
    * PersonaStore, so we need to keep a reference ourselves so we don't get
@@ -541,12 +538,24 @@ individual_personas_changed_cb (GObject *object,
   g_object_ref (self);
 
   /* Remove the old personas. */
-  for (l = removed; l != NULL; l = l->next)
-    remove_persona_and_disconnect (self, FOLKS_PERSONA (l->data));
+  iter = gee_iterable_iterator (GEE_ITERABLE (removed));
+  while (gee_iterator_next (iter))
+    {
+      FolksPersona *persona = gee_iterator_get (iter);
+      remove_persona_and_disconnect (self, persona);
+      g_clear_object (&persona);
+    }
+  g_clear_object (&iter);
 
   /* Add each of the new personas to the tree model */
-  for (l = added; l != NULL; l = l->next)
-    add_persona_and_connect (self, FOLKS_PERSONA (l->data));
+  iter = gee_iterable_iterator (GEE_ITERABLE (added));
+  while (gee_iterator_next (iter))
+    {
+      FolksPersona *persona = gee_iterator_get (iter);
+      add_persona_and_connect (self, persona);
+      g_clear_object (&persona);
+    }
+  g_clear_object (&iter);
 
   g_object_unref (self);
 }
@@ -563,8 +572,8 @@ sort_personas (FolksPersona *persona_a,
 
   /* alias */
   ret_val = g_utf8_collate (
-      folks_aliasable_get_alias (FOLKS_ALIASABLE (persona_a)),
-      folks_aliasable_get_alias (FOLKS_ALIASABLE (persona_b)));
+      folks_alias_details_get_alias (FOLKS_ALIAS_DETAILS (persona_a)),
+      folks_alias_details_get_alias (FOLKS_ALIAS_DETAILS (persona_b)));
 
   if (ret_val != 0)
     goto out;
@@ -588,14 +597,14 @@ sort_personas (FolksPersona *persona_a,
 
   /* protocol */
   ret_val = strcmp (tp_account_get_protocol (account_a),
-        tp_account_get_protocol (account_a));
+        tp_account_get_protocol (account_b));
 
   if (ret_val != 0)
     goto out;
 
   /* account ID */
   ret_val = strcmp (tp_proxy_get_object_path (account_a),
-        tp_proxy_get_object_path (account_a));
+        tp_proxy_get_object_path (account_b));
 
 out:
   return ret_val;
@@ -629,9 +638,10 @@ state_sort_func (GtkTreeModel *model,
    * the presences.
    */
   ret_val = -tp_connection_presence_type_cmp_availability (
-      folks_presence_owner_get_presence_type (FOLKS_PRESENCE_OWNER (persona_a)),
-      folks_presence_owner_get_presence_type (
-          FOLKS_PRESENCE_OWNER (persona_b)));
+      folks_presence_details_get_presence_type (
+          FOLKS_PRESENCE_DETAILS (persona_a)),
+      folks_presence_details_get_presence_type (
+          FOLKS_PRESENCE_DETAILS (persona_b)));
 
   if (ret_val == 0) {
     /* Fallback: compare by name et al. */
@@ -997,15 +1007,22 @@ empathy_persona_store_set_individual (EmpathyPersonaStore *self,
   /* Remove the old individual */
   if (priv->individual != NULL)
     {
-      GList *personas, *l;
+      GeeSet *personas;
+      GeeIterator *iter;
 
       g_signal_handlers_disconnect_by_func (priv->individual,
           (GCallback) individual_personas_changed_cb, self);
 
       /* Disconnect from and remove all personas belonging to this individual */
       personas = folks_individual_get_personas (priv->individual);
-      for (l = personas; l != NULL; l = l->next)
-        remove_persona_and_disconnect (self, FOLKS_PERSONA (l->data));
+      iter = gee_iterable_iterator (GEE_ITERABLE (personas));
+      while (gee_iterator_next (iter))
+        {
+          FolksPersona *persona = gee_iterator_get (iter);
+          remove_persona_and_disconnect (self, persona);
+          g_clear_object (&persona);
+        }
+      g_clear_object (&iter);
 
       g_object_unref (priv->individual);
     }
@@ -1015,7 +1032,8 @@ empathy_persona_store_set_individual (EmpathyPersonaStore *self,
   /* Add the new individual */
   if (individual != NULL)
     {
-      GList *personas, *l;
+      GeeSet *personas;
+      GeeIterator *iter;
 
       g_object_ref (individual);
 
@@ -1023,9 +1041,16 @@ empathy_persona_store_set_individual (EmpathyPersonaStore *self,
           (GCallback) individual_personas_changed_cb, self);
 
       /* Add pre-existing Personas */
+
       personas = folks_individual_get_personas (individual);
-      for (l = personas; l != NULL; l = l->next)
-        add_persona_and_connect (self, FOLKS_PERSONA (l->data));
+      iter = gee_iterable_iterator (GEE_ITERABLE (personas));
+      while (gee_iterator_next (iter))
+        {
+          FolksPersona *persona = gee_iterator_get (iter);
+          add_persona_and_connect (self, persona);
+          g_clear_object (&persona);
+        }
+      g_clear_object (&iter);
     }
 
   g_object_notify (G_OBJECT (self), "individual");
