@@ -28,14 +28,15 @@
 #include <telepathy-glib/account-manager.h>
 #include <telepathy-glib/interfaces.h>
 #include <telepathy-glib/util.h>
+#include <telepathy-yell/telepathy-yell.h>
 
 #include <telepathy-logger/log-manager.h>
 
 #include <folks/folks.h>
 #include <folks/folks-telepathy.h>
 
-#ifdef HAVE_GEOCLUE
-#include <geoclue/geoclue-geocode.h>
+#ifdef HAVE_GEOCODE
+#include <geocode-glib/geocode-glib.h>
 #endif
 
 #include "empathy-contact.h"
@@ -69,7 +70,7 @@ typedef struct {
    * more fields by searching the address using geoclue.
    */
   GHashTable *location;
-  GHashTable *groups;
+  GeeHashSet *groups;
   gchar **client_types;
 } EmpathyContactPriv;
 
@@ -79,7 +80,7 @@ static void contact_get_property (GObject *object, guint param_id,
 static void contact_set_property (GObject *object, guint param_id,
     const GValue *value, GParamSpec *pspec);
 
-#ifdef HAVE_GEOCLUE
+#ifdef HAVE_GEOCODE
 static void update_geocode (EmpathyContact *contact);
 #endif
 
@@ -189,14 +190,12 @@ contact_dispose (GObject *object)
 {
   EmpathyContactPriv *priv = GET_PRIV (object);
 
-  if (priv->tp_contact)
+  if (priv->tp_contact != NULL)
     {
-      g_hash_table_remove (contacts_table, priv->tp_contact);
       g_signal_handlers_disconnect_by_func (priv->tp_contact,
           tp_contact_notify_cb, object);
-      g_object_unref (priv->tp_contact);
     }
-  priv->tp_contact = NULL;
+  tp_clear_object (&priv->tp_contact);
 
   if (priv->account)
     g_object_unref (priv->account);
@@ -424,8 +423,7 @@ contact_finalize (GObject *object)
 
   DEBUG ("finalize: %p", object);
 
-  if (priv->groups != NULL)
-    g_hash_table_destroy (priv->groups);
+  g_clear_object (&priv->groups);
   g_free (priv->alias);
   g_free (priv->id);
   g_strfreev (priv->client_types);
@@ -631,14 +629,48 @@ contact_set_property (GObject *object,
     };
 }
 
+static void
+remove_tp_contact (gpointer data,
+    GObject *object)
+{
+  g_hash_table_remove (contacts_table, data);
+}
+
 static EmpathyContact *
 empathy_contact_new (TpContact *tp_contact)
 {
+  EmpathyContact *retval;
+
   g_return_val_if_fail (TP_IS_CONTACT (tp_contact), NULL);
 
-  return g_object_new (EMPATHY_TYPE_CONTACT,
+  retval = g_object_new (EMPATHY_TYPE_CONTACT,
       "tp-contact", tp_contact,
       NULL);
+
+  g_object_weak_ref (G_OBJECT (retval), remove_tp_contact, tp_contact);
+
+  return retval;
+}
+
+typedef struct
+{
+  TplEntity *entity;
+  TpAccount *account;
+} FindContactData;
+
+static gboolean
+contact_is_tpl_entity (gpointer key,
+    gpointer value,
+    gpointer user_data)
+{
+  EmpathyContact *contact = value;
+  FindContactData *data = user_data;
+
+  return !tp_strdiff (empathy_contact_get_id (contact),
+              tpl_entity_get_identifier (data->entity)) &&
+         !tp_strdiff (tp_proxy_get_object_path (data->account),
+              tp_proxy_get_object_path (
+                  empathy_contact_get_account (contact)));
 }
 
 EmpathyContact *
@@ -647,17 +679,49 @@ empathy_contact_from_tpl_contact (TpAccount *account,
 {
   EmpathyContact *retval;
   gboolean is_user;
+  EmpathyContact *existing_contact = NULL;
 
   g_return_val_if_fail (TPL_IS_ENTITY (tpl_entity), NULL);
 
-  is_user = (TPL_ENTITY_SELF == tpl_entity_get_entity_type (tpl_entity));
+  if (contacts_table != NULL)
+    {
+      FindContactData data;
 
-  retval = g_object_new (EMPATHY_TYPE_CONTACT,
-      "id", tpl_entity_get_identifier (tpl_entity),
-      "alias", tpl_entity_get_alias (tpl_entity),
-      "account", account,
-      "is-user", is_user,
-      NULL);
+      data.entity = tpl_entity;
+      data.account = account;
+
+      existing_contact = g_hash_table_find (contacts_table,
+        contact_is_tpl_entity, &data);
+    }
+
+  if (existing_contact != NULL)
+    {
+      EmpathyContactPriv *priv;
+
+      retval = g_object_new (EMPATHY_TYPE_CONTACT,
+          "tp-contact", empathy_contact_get_tp_contact (existing_contact),
+          NULL);
+
+      priv = GET_PRIV (retval);
+
+      /* contact_set_property() calls empathy_contact_set_alias(), which
+       * tries to set the alias on the FolksPersona, but we don't want to
+       * do that when creating an EmpathyContact from a TplEntity. So just
+       * set priv->alias instead of passing it to g_object_new() instead. */
+      g_free (priv->alias);
+      priv->alias = g_strdup (tpl_entity_get_alias (tpl_entity));
+    }
+  else
+    {
+      is_user = (TPL_ENTITY_SELF == tpl_entity_get_entity_type (tpl_entity));
+
+      retval = g_object_new (EMPATHY_TYPE_CONTACT,
+          "id", tpl_entity_get_identifier (tpl_entity),
+          "alias", tpl_entity_get_alias (tpl_entity),
+          "account", account,
+          "is-user", is_user,
+          NULL);
+    }
 
   if (!EMP_STR_EMPTY (tpl_entity_get_avatar_token (tpl_entity)))
     contact_load_avatar_cache (retval,
@@ -697,16 +761,16 @@ const gchar *
 empathy_contact_get_alias (EmpathyContact *contact)
 {
   EmpathyContactPriv *priv;
-  const gchar        *alias;
+  const gchar        *alias = NULL;
 
   g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
 
   priv = GET_PRIV (contact);
 
-  if (priv->tp_contact != NULL)
-    alias = tp_contact_get_alias (priv->tp_contact);
-  else
+  if (!EMP_STR_EMPTY (priv->alias))
     alias = priv->alias;
+  else if (priv->tp_contact != NULL)
+    alias = tp_contact_get_alias (priv->tp_contact);
 
   if (!EMP_STR_EMPTY (alias))
     return alias;
@@ -789,12 +853,11 @@ empathy_contact_change_group (EmpathyContact *contact, const gchar *group,
    * does */
   if (priv->groups == NULL)
     {
-      priv->groups = g_hash_table_new_full (g_str_hash, g_str_equal, g_free,
-          NULL);
+      priv->groups = gee_hash_set_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup,
+          g_free, g_str_hash, g_str_equal);
     }
 
-  g_hash_table_insert (priv->groups, g_strdup (group),
-      GUINT_TO_POINTER (is_member));
+  gee_collection_add (GEE_COLLECTION (priv->groups), group);
 }
 
 EmpathyAvatar *
@@ -877,13 +940,16 @@ empathy_contact_get_persona (EmpathyContact *contact)
 
       for (l = individuals; l != NULL; l = l->next)
         {
-          GList *personas, *j;
           FolksIndividual *individual = FOLKS_INDIVIDUAL (l->data);
+          GeeSet *personas;
+          GeeIterator *iter;
+          gboolean persona_found = FALSE;
 
           personas = folks_individual_get_personas (individual);
-          for (j = personas; j != NULL; j = j->next)
+          iter = gee_iterable_iterator (GEE_ITERABLE (personas));
+          while (!persona_found && gee_iterator_next (iter))
             {
-              TpfPersona *persona = j->data;
+              TpfPersona *persona = gee_iterator_get (iter);
 
               if (empathy_folks_persona_is_interesting (FOLKS_PERSONA (persona)))
                 {
@@ -894,13 +960,14 @@ empathy_contact_get_persona (EmpathyContact *contact)
                       /* Found the right persona */
                       empathy_contact_set_persona (contact,
                           (FolksPersona *) persona);
-                      goto finished;
+                      persona_found = TRUE;
                     }
+                  g_clear_object (&persona);
                 }
             }
+          g_clear_object (&iter);
         }
 
-finished:
       g_list_free (individuals);
       g_object_unref (manager);
     }
@@ -945,8 +1012,8 @@ empathy_contact_set_persona (EmpathyContact *contact,
   if (priv->groups != NULL)
     {
       folks_group_details_set_groups (FOLKS_GROUP_DETAILS (persona),
-          priv->groups);
-      g_hash_table_destroy (priv->groups);
+          GEE_SET (priv->groups));
+      g_object_unref (priv->groups);
       priv->groups = NULL;
     }
 }
@@ -1097,6 +1164,18 @@ empathy_contact_get_status (EmpathyContact *contact)
       empathy_contact_get_presence (contact));
 }
 
+gboolean
+empathy_contact_can_sms (EmpathyContact *contact)
+{
+  EmpathyContactPriv *priv;
+
+  g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), FALSE);
+
+  priv = GET_PRIV (contact);
+
+  return priv->capabilities & EMPATHY_CAPABILITIES_SMS;
+}
+
 gboolean
 empathy_contact_can_voip (EmpathyContact *contact)
 {
@@ -1189,6 +1268,9 @@ empathy_contact_can_do_action (EmpathyContact *self,
       case EMPATHY_ACTION_CHAT:
         sensitivity = TRUE;
         break;
+      case EMPATHY_ACTION_SMS:
+        sensitivity = empathy_contact_can_sms (self);
+        break;
       case EMPATHY_ACTION_AUDIO_CALL:
         sensitivity = empathy_contact_can_voip_audio (self);
         break;
@@ -1267,17 +1349,16 @@ contact_load_avatar_cache (EmpathyContact *contact,
         }
     }
 
-  if (data)
+  if (data != NULL)
     {
       DEBUG ("Avatar loaded from %s", filename);
       avatar = empathy_avatar_new ((guchar *) data, len, NULL, filename);
       contact_set_avatar (contact, avatar);
       empathy_avatar_unref (avatar);
     }
-  else
-    {
-      g_free (filename);
-    }
+
+  g_free (data);
+  g_free (filename);
 
   return data != NULL;
 }
@@ -1304,24 +1385,23 @@ empathy_avatar_get_type (void)
  * @format: the mime type of the avatar image
  * @filename: the filename where the avatar is stored in cache
  *
- * Create a #EmpathyAvatar from the provided data. This function takes the
- * ownership of @data, @format and @filename.
+ * Create a #EmpathyAvatar from the provided data.
  *
  * Returns: a new #EmpathyAvatar
  */
 EmpathyAvatar *
-empathy_avatar_new (guchar *data,
+empathy_avatar_new (const guchar *data,
                     gsize len,
-                    gchar *format,
-                    gchar *filename)
+                    const gchar *format,
+                    const gchar *filename)
 {
   EmpathyAvatar *avatar;
 
   avatar = g_slice_new0 (EmpathyAvatar);
-  avatar->data = data;
+  avatar->data = g_memdup (data, len);
   avatar->len = len;
-  avatar->format = format;
-  avatar->filename = filename;
+  avatar->format = g_strdup (format);
+  avatar->filename = g_strdup (filename);
   avatar->refcount = 1;
 
   return avatar;
@@ -1425,7 +1505,7 @@ empathy_contact_set_location (EmpathyContact *contact,
     g_hash_table_unref (priv->location);
 
   priv->location = g_hash_table_ref (location);
-#ifdef HAVE_GEOCLUE
+#ifdef HAVE_GEOCODE
   update_geocode (contact);
 #endif
   g_object_notify (G_OBJECT (contact), "location");
@@ -1493,41 +1573,37 @@ empathy_contact_equal (gconstpointer contact1,
   return FALSE;
 }
 
-#ifdef HAVE_GEOCLUE
-#define GEOCODE_SERVICE "org.freedesktop.Geoclue.Providers.Yahoo"
-#define GEOCODE_PATH "/org/freedesktop/Geoclue/Providers/Yahoo"
-
-/* This callback is called by geoclue when it found a position
+#ifdef HAVE_GEOCODE
+/* This callback is called by geocode-glib when it found a position
  * for the given address.  A position is necessary for a contact
  * to show up on the map
  */
 static void
-geocode_cb (GeoclueGeocode *geocode,
-    GeocluePositionFields fields,
-    double latitude,
-    double longitude,
-    double altitude,
-    GeoclueAccuracy *accuracy,
-    GError *error,
-    gpointer contact)
+geocode_cb (GObject *source,
+    GAsyncResult *result,
+    gpointer user_data)
 {
+  EmpathyContact *contact = user_data;
   EmpathyContactPriv *priv = GET_PRIV (contact);
+  GError *error = NULL;
   GHashTable *new_location;
+  GHashTable *resolved;
+  gdouble latitude, longitude;
 
   if (priv->location == NULL)
     goto out;
 
-  if (error != NULL)
+  resolved = geocode_object_resolve_finish (GEOCODE_OBJECT (source), result,
+      &error);
+
+  if (resolved == NULL)
     {
-      DEBUG ("Error geocoding location : %s", error->message);
+      DEBUG ("Failed to resolve geocode: %s", error->message);
+      g_error_free (error);
       goto out;
     }
 
-  /* No need to change location if we didn't find the position */
-  if (!(fields & GEOCLUE_POSITION_FIELDS_LATITUDE))
-    goto out;
-
-  if (!(fields & GEOCLUE_POSITION_FIELDS_LONGITUDE))
+  if (!geocode_object_get_coords (resolved, &longitude, &latitude))
     goto out;
 
   new_location = tp_asv_new (
@@ -1543,43 +1619,20 @@ geocode_cb (GeoclueGeocode *geocode,
   tp_g_hash_table_update (new_location, priv->location,
       (GBoxedCopyFunc) g_strdup, (GBoxedCopyFunc) tp_g_value_slice_dup);
 
-  /* Set the altitude only if it wasn't defined before */
-  if (fields & GEOCLUE_POSITION_FIELDS_ALTITUDE &&
-      g_hash_table_lookup (new_location, EMPATHY_LOCATION_ALT) == NULL)
-    {
-      tp_asv_set_double (new_location, g_strdup (EMPATHY_LOCATION_ALT),
-          altitude);
-      DEBUG ("\t - Altitude: %f", altitude);
-    }
-
   /* Don't change the accuracy as we used an address to get this position */
   g_hash_table_unref (priv->location);
   priv->location = new_location;
-  g_object_notify (contact, "location");
+  g_object_notify ((GObject *) contact, "location");
+
 out:
-  g_object_unref (geocode);
+  tp_clear_pointer (&result, g_hash_table_unref);
   g_object_unref (contact);
 }
 
-static gchar *
-get_dup_string (GHashTable *location,
-    gchar *key)
-{
-  GValue *value;
-
-  value = g_hash_table_lookup (location, key);
-  if (value != NULL)
-    return g_value_dup_string (value);
-
-  return NULL;
-}
-
 static void
 update_geocode (EmpathyContact *contact)
 {
-  static GeoclueGeocode *geocode;
-  gchar *str;
-  GHashTable *address;
+  GeocodeObject *geocode;
   GHashTable *location;
 
   location = empathy_contact_get_location (contact);
@@ -1591,75 +1644,12 @@ update_geocode (EmpathyContact *contact)
       g_hash_table_lookup (location, EMPATHY_LOCATION_LON) != NULL)
     return;
 
-  if (geocode == NULL)
-    {
-      geocode = geoclue_geocode_new (GEOCODE_SERVICE, GEOCODE_PATH);
-      g_object_add_weak_pointer (G_OBJECT (geocode), (gpointer *) &geocode);
-    }
-  else
-    {
-      g_object_ref (geocode);
-    }
+  geocode = geocode_object_new_for_params (location);
 
-  address = geoclue_address_details_new ();
+  geocode_object_resolve_async (geocode, NULL, geocode_cb,
+      g_object_ref (contact));
 
-  str = get_dup_string (location, EMPATHY_LOCATION_COUNTRY_CODE);
-  if (str != NULL)
-    {
-      g_hash_table_insert (address,
-        g_strdup (GEOCLUE_ADDRESS_KEY_COUNTRYCODE), str);
-      DEBUG ("\t - countrycode: %s", str);
-    }
-
-  str = get_dup_string (location, EMPATHY_LOCATION_COUNTRY);
-  if (str != NULL)
-    {
-      g_hash_table_insert (address,
-        g_strdup (GEOCLUE_ADDRESS_KEY_COUNTRY), str);
-      DEBUG ("\t - country: %s", str);
-    }
-
-  str = get_dup_string (location, EMPATHY_LOCATION_POSTAL_CODE);
-  if (str != NULL)
-    {
-      g_hash_table_insert (address,
-        g_strdup (GEOCLUE_ADDRESS_KEY_POSTALCODE), str);
-      DEBUG ("\t - postalcode: %s", str);
-    }
-
-  str = get_dup_string (location, EMPATHY_LOCATION_REGION);
-  if (str != NULL)
-    {
-      g_hash_table_insert (address,
-        g_strdup (GEOCLUE_ADDRESS_KEY_REGION), str);
-      DEBUG ("\t - region: %s", str);
-    }
-
-  str = get_dup_string (location, EMPATHY_LOCATION_LOCALITY);
-  if (str != NULL)
-    {
-      g_hash_table_insert (address,
-        g_strdup (GEOCLUE_ADDRESS_KEY_LOCALITY), str);
-      DEBUG ("\t - locality: %s", str);
-    }
-
-  str = get_dup_string (location, EMPATHY_LOCATION_STREET);
-  if (str != NULL)
-    {
-      g_hash_table_insert (address,
-        g_strdup (GEOCLUE_ADDRESS_KEY_STREET), str);
-      DEBUG ("\t - street: %s", str);
-    }
-
-  if (g_hash_table_size (address) > 0)
-    {
-      g_object_ref (contact);
-
-      geoclue_geocode_address_to_position_async (geocode, address,
-          geocode_cb, contact);
-    }
-
-  g_hash_table_unref (address);
+  g_object_unref (geocode);
 }
 #endif
 
@@ -1729,6 +1719,35 @@ tp_caps_to_capabilities (TpCapabilities *caps)
                     TP_PROP_CHANNEL_TYPE_STREAMED_MEDIA_INITIAL_VIDEO, NULL))
             capabilities |= EMPATHY_CAPABILITIES_VIDEO;
         }
+      else if (!tp_strdiff (chan_type, TP_IFACE_CHANNEL_TYPE_TEXT))
+        {
+          if (tp_asv_get_boolean (fixed_prop,
+                TP_PROP_CHANNEL_INTERFACE_SMS_SMS_CHANNEL, NULL))
+            capabilities |= EMPATHY_CAPABILITIES_SMS;
+        }
+      else if (!tp_strdiff (chan_type,
+        TPY_IFACE_CHANNEL_TYPE_CALL))
+        {
+          guint j;
+
+          if (tp_asv_get_boolean (fixed_prop,
+              TPY_PROP_CHANNEL_TYPE_CALL_INITIAL_AUDIO, NULL))
+            capabilities |= EMPATHY_CAPABILITIES_AUDIO;
+
+          if (tp_asv_get_boolean (fixed_prop,
+              TPY_PROP_CHANNEL_TYPE_CALL_INITIAL_VIDEO, NULL))
+            capabilities |= EMPATHY_CAPABILITIES_VIDEO;
+
+          for (j = 0; allowed_prop[j] != NULL; j++)
+            {
+              if (!tp_strdiff (allowed_prop[j],
+                    TPY_PROP_CHANNEL_TYPE_CALL_INITIAL_AUDIO))
+                capabilities |= EMPATHY_CAPABILITIES_AUDIO;
+              else if (!tp_strdiff (allowed_prop[j],
+                    TPY_PROP_CHANNEL_TYPE_CALL_INITIAL_VIDEO))
+                capabilities |= EMPATHY_CAPABILITIES_VIDEO;
+            }
+        }
     }
 
   return capabilities;
@@ -1762,12 +1781,17 @@ contact_set_avatar_from_tp_contact (EmpathyContact *contact)
       EmpathyAvatar *avatar;
       gchar *data;
       gsize len;
+      gchar *path;
 
       g_file_load_contents (file, NULL, &data, &len, NULL, NULL);
-      avatar = empathy_avatar_new ((guchar *) data, len, g_strdup (mime),
-          g_file_get_path (file));
+      path = g_file_get_path (file);
+
+      avatar = empathy_avatar_new ((guchar *) data, len, mime, path);
+
       contact_set_avatar (contact, avatar);
       empathy_avatar_unref (avatar);
+      g_free (path);
+      g_free (data);
     }
   else
     {
@@ -1959,35 +1983,39 @@ EmpathyContact *
 empathy_contact_dup_best_for_action (FolksIndividual *individual,
     EmpathyActionType action_type)
 {
-  GList *personas, *contacts, *l;
+  GeeSet *personas;
+  GeeIterator *iter;
+  GList *contacts;
   EmpathyContact *best_contact = NULL;
 
   /* Build a list of EmpathyContacts that we can sort */
   personas = folks_individual_get_personas (individual);
   contacts = NULL;
 
-  for (l = personas; l != NULL; l = l->next)
+  iter = gee_iterable_iterator (GEE_ITERABLE (personas));
+  while (gee_iterator_next (iter))
     {
+      FolksPersona *persona = gee_iterator_get (iter);
       TpContact *tp_contact;
-      EmpathyContact *contact;
+      EmpathyContact *contact = NULL;
 
-      if (!empathy_folks_persona_is_interesting (FOLKS_PERSONA (l->data)))
-        continue;
+      if (!empathy_folks_persona_is_interesting (persona))
+        goto while_finish;
 
-      tp_contact = tpf_persona_get_contact (TPF_PERSONA (l->data));
+      tp_contact = tpf_persona_get_contact (TPF_PERSONA (persona));
       contact = empathy_contact_dup_from_tp_contact (tp_contact);
-      empathy_contact_set_persona (contact, FOLKS_PERSONA (l->data));
+      empathy_contact_set_persona (contact, FOLKS_PERSONA (persona));
 
       /* Only choose the contact if they're actually capable of the specified
        * action. */
-      if (!empathy_contact_can_do_action (contact, action_type))
-        {
-          g_object_unref (contact);
-          continue;
-        }
+      if (empathy_contact_can_do_action (contact, action_type))
+        contacts = g_list_prepend (contacts, g_object_ref (contact));
 
-      contacts = g_list_prepend (contacts, contact);
+while_finish:
+      g_clear_object (&contact);
+      g_clear_object (&persona);
     }
+  g_clear_object (&iter);
 
   /* Sort the contacts by some heuristic based on the action type, then take
    * the top contact. */
