@@ -26,6 +26,8 @@
 
 #include <string.h>
 
+#include <glib/gi18n-lib.h>
+
 #include <telepathy-glib/util.h>
 #include <telepathy-glib/account.h>
 #include <telepathy-glib/account-manager.h>
@@ -33,6 +35,9 @@
 #include <telepathy-logger/entity.h>
 #include <telepathy-logger/event.h>
 #include <telepathy-logger/text-event.h>
+#ifdef HAVE_CALL_LOGS
+# include <telepathy-logger/call-event.h>
+#endif
 
 #include "empathy-message.h"
 #include "empathy-utils.h"
@@ -40,6 +45,7 @@
 
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyMessage)
 typedef struct {
+	TpMessage *tp_message;
 	TpChannelTextMessageType  type;
 	EmpathyContact           *sender;
 	EmpathyContact           *receiver;
@@ -73,6 +79,7 @@ enum {
 	PROP_IS_BACKLOG,
 	PROP_INCOMING,
 	PROP_FLAGS,
+	PROP_TP_MESSAGE,
 };
 
 static void
@@ -153,6 +160,15 @@ empathy_message_class_init (EmpathyMessageClass *class)
 							       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS |
 							       G_PARAM_CONSTRUCT_ONLY));
 
+	g_object_class_install_property (object_class,
+					 PROP_TP_MESSAGE,
+					 g_param_spec_object ("tp-message",
+							       "TpMessage",
+							       "The TpMessage of this message",
+							       TP_TYPE_MESSAGE,
+							       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS |
+							       G_PARAM_CONSTRUCT_ONLY));
+
 	g_type_class_add_private (object_class, sizeof (EmpathyMessagePriv));
 
 }
@@ -181,6 +197,10 @@ empathy_message_finalize (GObject *object)
 		g_object_unref (priv->receiver);
 	}
 
+	if (priv->tp_message) {
+		g_object_unref (priv->tp_message);
+	}
+
 	g_free (priv->body);
 
 	G_OBJECT_CLASS (empathy_message_parent_class)->finalize (object);
@@ -221,6 +241,9 @@ message_get_property (GObject    *object,
 	case PROP_FLAGS:
 		g_value_set_uint (value, priv->flags);
 		break;
+	case PROP_TP_MESSAGE:
+		g_value_set_object (value, priv->tp_message);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -267,6 +290,9 @@ message_set_property (GObject      *object,
 	case PROP_FLAGS:
 		priv->flags = g_value_get_uint (value);
 		break;
+	case PROP_TP_MESSAGE:
+		priv->tp_message = g_value_dup_object (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -283,6 +309,7 @@ empathy_message_from_tpl_log_event (TplEvent *logevent)
 	TplEntity *sender = NULL;
 	gchar *body= NULL;
 	EmpathyContact *contact;
+	TpChannelTextMessageType type = TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL;
 
 	g_return_val_if_fail (TPL_IS_EVENT (logevent), NULL);
 
@@ -302,24 +329,36 @@ empathy_message_from_tpl_log_event (TplEvent *logevent)
 			tpl_event_get_account_path (logevent));
 	g_object_unref (acc_man);
 
-	/* TODO Currently only TplTextEvent exists as subclass of TplEvent, in
-	 * future more TplEvent will exist and EmpathyMessage should probably
-	 * be enhanced to support other types of log entries (ie TplCallEvent).
-	 *
-	 * For now we just check (simply) that we are dealing with the only supported type,
-	 * then there will be a if/then/else or switch handling all the supported
-	 * cases.
-	 */
-	if (!TPL_IS_TEXT_EVENT (logevent))
+	if (TPL_IS_TEXT_EVENT (logevent)) {
+		body = g_strdup (tpl_text_event_get_message (
+			TPL_TEXT_EVENT (logevent)));
+
+		type = tpl_text_event_get_message_type (TPL_TEXT_EVENT (logevent));
+	}
+#ifdef HAVE_CALL_LOGS
+	else if (TPL_IS_CALL_EVENT (logevent)) {
+		TplCallEvent *call = TPL_CALL_EVENT (logevent);
+		if (tpl_call_event_get_end_reason (call) == TPL_CALL_END_REASON_NO_ANSWER)
+			body = g_strdup_printf (_("Missed call from %s"),
+				tpl_entity_get_alias (tpl_event_get_sender (logevent)));
+		else if (tpl_entity_get_entity_type (tpl_event_get_sender (logevent)) == TPL_ENTITY_SELF)
+			body = g_strdup_printf (_("Called %s"),
+				tpl_entity_get_alias (tpl_event_get_receiver (logevent)));
+		else
+			body = g_strdup_printf (_("Call from %s"),
+				tpl_entity_get_alias (tpl_event_get_sender (logevent)));
+	}
+#endif
+	else {
+		/* Unknown event type */
 		return NULL;
+	}
 
-	body = g_strdup (tpl_text_event_get_message (
-				TPL_TEXT_EVENT (logevent)));
 	receiver = tpl_event_get_receiver (logevent);
 	sender = tpl_event_get_sender (logevent);
 
 	retval = g_object_new (EMPATHY_TYPE_MESSAGE,
-		"type", tpl_text_event_get_message_type (TPL_TEXT_EVENT (logevent)),
+		"type", type,
 		"body", body,
 		"is-backlog", TRUE,
 		"timestamp", tpl_event_get_timestamp (logevent),
@@ -342,6 +381,18 @@ empathy_message_from_tpl_log_event (TplEvent *logevent)
 	return retval;
 }
 
+TpMessage *
+empathy_message_get_tp_message (EmpathyMessage *message)
+{
+	EmpathyMessagePriv *priv;
+
+	g_return_val_if_fail (EMPATHY_IS_MESSAGE (message), NULL);
+
+	priv = GET_PRIV (message);
+
+	return priv->tp_message;
+}
+
 TpChannelTextMessageType
 empathy_message_get_tptype (EmpathyMessage *message)
 {
@@ -563,16 +614,6 @@ empathy_message_type_to_str (TpChannelTextMessageType type)
 	}
 }
 
-guint
-empathy_message_get_id (EmpathyMessage *message)
-{
-	EmpathyMessagePriv *priv = GET_PRIV (message);
-
-	g_return_val_if_fail (EMPATHY_IS_MESSAGE (message), 0);
-
-	return priv->id;
-}
-
 gboolean
 empathy_message_is_incoming (EmpathyMessage *message)
 {
@@ -618,10 +659,8 @@ empathy_message_new_from_tp_message (TpMessage *tp_msg,
 				     gboolean incoming)
 {
 	EmpathyMessage *message;
-	EmpathyMessagePriv *priv;
 	gchar *body;
 	TpChannelTextMessageFlags flags;
-	guint id;
 
 	g_return_val_if_fail (TP_IS_MESSAGE (tp_msg), NULL);
 
@@ -634,17 +673,9 @@ empathy_message_new_from_tp_message (TpMessage *tp_msg,
 		"flags", flags,
 		"is-backlog", flags & TP_CHANNEL_TEXT_MESSAGE_FLAG_SCROLLBACK,
 		"incoming", incoming,
+		"tp-message", tp_msg,
 		NULL);
 
-	priv = GET_PRIV (message);
-
-	/* FIXME: this is pretty low level, ideally we shouldn't have to use the
-	 * ID directly but we don't use TpTextChannel's ack API everywhere yet. */
-	id = tp_asv_get_uint32 (tp_message_peek (tp_msg, 0),
-		"pending-message-id", NULL);
-
-	priv->id = id;
-
 	g_free (body);
 	return message;
 }
