@@ -42,14 +42,15 @@ G_BEGIN_DECLS
 
 typedef struct _EmpathyTpChat      EmpathyTpChat;
 typedef struct _EmpathyTpChatClass EmpathyTpChatClass;
+typedef struct _EmpathyTpChatPrivate EmpathyTpChatPrivate;
 
 struct _EmpathyTpChat {
-	GObject parent;
-	gpointer priv;
+	TpTextChannel parent;
+	EmpathyTpChatPrivate *priv;
 };
 
 struct _EmpathyTpChatClass {
-	GObjectClass parent_class;
+	TpTextChannelClass parent_class;
 };
 
 typedef struct {
@@ -65,19 +66,21 @@ typedef enum {
 	EMPATHY_DELIVERY_STATUS_ACCEPTED
 } EmpathyDeliveryStatus;
 
+#define EMPATHY_TP_CHAT_FEATURE_READY empathy_tp_chat_get_feature_ready ()
+GQuark empathy_tp_chat_get_feature_ready (void) G_GNUC_CONST;
+
 GType          empathy_tp_chat_get_type             (void) G_GNUC_CONST;
+
 EmpathyTpChat *empathy_tp_chat_new                  (TpAccount *account,
-						     TpChannel *channel);
+						     TpConnection *connection,
+						     const gchar *object_path,
+						     const GHashTable *immutable_properties);
+
 const gchar *  empathy_tp_chat_get_id               (EmpathyTpChat      *chat);
 EmpathyContact *empathy_tp_chat_get_remote_contact   (EmpathyTpChat      *chat);
-TpChannel *    empathy_tp_chat_get_channel          (EmpathyTpChat      *chat);
 TpAccount    * empathy_tp_chat_get_account          (EmpathyTpChat      *chat);
-TpConnection * empathy_tp_chat_get_connection       (EmpathyTpChat      *chat);
-gboolean       empathy_tp_chat_is_ready             (EmpathyTpChat      *chat);
 void           empathy_tp_chat_send                 (EmpathyTpChat      *chat,
 						     TpMessage     *message);
-void           empathy_tp_chat_set_state            (EmpathyTpChat      *chat,
-						     TpChannelChatState  state);
 void           empathy_tp_chat_set_property         (EmpathyTpChat      *chat,
 						     const gchar        *name,
 						     const GValue       *value);
@@ -118,8 +121,6 @@ TpChannelChatState
 
 EmpathyContact * empathy_tp_chat_get_self_contact   (EmpathyTpChat      *self);
 
-gboolean       empathy_tp_chat_is_sms_channel       (EmpathyTpChat      *chat);
-
 G_END_DECLS
 
 #endif /* __EMPATHY_TP_CHAT_H__ */
