@@ -24,12 +24,10 @@
 
 gboolean empathy_accounts_has_non_salut_accounts (TpAccountManager *manager);
 
+gboolean empathy_accounts_has_accounts (TpAccountManager *manager);
+
 void empathy_accounts_show_accounts_ui (TpAccountManager *manager,
     TpAccount *account,
     GCallback window_destroyed_cb);
 
-void empathy_accounts_import (TpAccountManager *account_mgr,
-    EmpathyConnectionManagers *cm_mgr);
-
-
 #endif /* __EMPATHY_ACCOUNTS_COMMON_H__ */
